
import sfml as sf
import math

TAILLE_PAR_DEFAUT = 800
NOIR = sf.graphics.Color.BLACK
BLANC = sf.graphics.Color.WHITE
ROUGE = sf.graphics.Color.RED
VERT = sf.graphics.Color.GREEN
BLEU = sf.graphics.Color.BLUE
JAUNE = sf.graphics.Color.YELLOW
MAGENTA = sf.graphics.Color.MAGENTA
TURQUOISE = sf.graphics.Color.CYAN
COULEUR_PAR_DEFAUT = VERT
HAUTEUR = 690

''' *********************************************************************
    canevas.py

    author Gabriel Girard
    date de creation: 16 janvier 2017

    \brief Cette classe permet de creer et manipuler un ecran
           d'affichage pour le jeu du tir au canon.

 ** *********************************************************************'''
class Canevas :
    def __init__(self) :
        DIM_X = 1200
        DIM_Y = 800
        #----------------------
        # Creation du canevas de dessin..
        #
        self.canevas = sf.graphics.RenderWindow(sf.window.VideoMode(DIM_X, DIM_Y), "Super jeu du tir au canon V. 0.1!")
        self.canevas.framerate_limit = 60

        #----------------
        # Creation de la cible
        #
        self.imageCible = sf.graphics.Image.from_file("images/tank2.png")
        self.textureCible = sf.graphics.Texture.from_file("images/tank2.png")
        self.cible = sf.graphics.Sprite(self.textureCible)
        self.cible.texture_rectangle = sf.graphics.Rectangle((150, 40), (150, 120))
        self.cible.scale(sf.system.Vector2(.3,.3))
        self.cible.position = sf.system.Vector2(600,HAUTEUR)
        #----------------------------------
        self.textureBoulet = sf.graphics.Texture.from_file("images/boulet.png")
        self.boulet = sf.graphics.Sprite(self.textureBoulet)
        self.boulet.texture_rectangle = sf.graphics.Rectangle((0, 200), (64, 64))
        self.boulet.scale(sf.system.Vector2(.3,.3))
        #------------------------
        # Creation de l'explosion....
        self.texture1 = sf.graphics.Texture.from_file("images/explosion.png")
        self.imageCourante= sf.graphics.Sprite(self.texture1)
        self.images = []
        self.images.append(sf.graphics.Rectangle((0, 0), (64, 64)))
        self.images.append(sf.graphics.Rectangle((64, 0), (64, 64)))
        self.images.append(sf.graphics.Rectangle((128, 0), (64, 64)))
        self.images.append(sf.graphics.Rectangle((196, 0), (64, 64)))
        self.images.append(sf.graphics.Rectangle((0, 64), (64, 64)))
        self.images.append(sf.graphics.Rectangle((64, 64), (64, 64)))
        self.images.append(sf.graphics.Rectangle((128, 64),(64, 64)))
        self.images.append(sf.graphics.Rectangle((196, 64), (64, 64)))
        self.images.append(sf.graphics.Rectangle((0, 128), (64, 64)))
        self.images.append(sf.graphics.Rectangle((64, 128), (64, 64)))
        self.images.append(sf.graphics.Rectangle((128, 128), (64, 64)))
        self.images.append(sf.graphics.Rectangle((196, 128), (64, 64)))
        self.images.append(sf.graphics.Rectangle((0, 196), (64, 64)))
        self.images.append(sf.graphics.Rectangle((64, 196), (64, 64)))
        self.images.append(sf.graphics.Rectangle((128, 196), (64, 64)))
        self.images.append(sf.graphics.Rectangle((196, 196), (64, 64)))

        # Creation dutexte
        self.font = sf.graphics.Font.from_file("images/arial.ttf")
        self.textePts = sf.graphics.Text("Points restants = 250", self.font, 30 )
        self.textePts.style = sf.graphics.Text.BOLD
        self.textePts.color = sf.graphics.Color.RED
        self.textePts.position = (800,0)
        self.texteCoups = sf.graphics.Text("Nombre de coups = 0", self.font, 30 )
        self.texteCoups.style = sf.graphics.Text.BOLD
        self.texteCoups.color = sf.graphics.Color.RED
        self.texteCoups.position = (800,40)

        self.canevas.clear(VERT)
        self.canevas.draw(self.textePts)
        self.canevas.draw(self.texteCoups)
        self.canevas.draw(self.cible)
        self.canevas.display()

        self.tailleX = DIM_X
        self.tailleY = DIM_Y

    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Destructeur de la classe Ecran.  Il ferme la fenetre.

    ** *********************************************************************'''
    def __del__(self):
        print("fin......................:")
        self.canevas.close()

    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Efface le contenu du canevas

    *********************************************************************'''
    def effacer(self):
        self.canevas.clear(VERT)


    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction calcule la position en x du boulet

    \param[in] angleRadian : l'angle du tir
    \param[in] vitesse : la vitesse initiale du boulet
    \param[in] temps : le temps passé depuis le tir
    *********************************************************************'''
    def calculerX(self, angleRadian, vitesse, temps):
        return  vitesse * math.cos(angleRadian) * temps


    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

     \brief Cette fonction calcule la position en y du boulet

    \param[in] angleRadian : l'angle du tir
    \param[in] vitesse : la vitesse initiale du boulet
    \param[in] temps : le temps passé depuis le tir
    *********************************************************************'''
    def calculerY(self, pos, angleRadian, vitesse, temps):
        return pos + (vitesse * math.sin(angleRadian) * temps) - (32.2 * temps*temps/2)


    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction place la cible sur l'écran.  On l'appelle
            aussitôt que la distance de la cible est connue.

    \param[in] distance : la distance de la cible

    <b>Sorties:</b>
        \li (\c ecran)  La fenetre d'affichage  (\c graphique)
        \li (\c ecran)  La cible   (\c graphique)
        \li (\c ecran)  Le nombre de points courant de la cible  (\c entier)
        \li (\c ecran)  Le nombre de coups tires (0)  (\c entier)
    *********************************************************************'''
    def placeCible(self, distance):
        # Réactive l'écran pour remettre les couleurs et enlever le "fading"
        for event in self.canevas.events:
            if type(event) is sf.window.CloseEvent :
                    self.canevas.close()
                    exit()
        self.cible.position = (sf.system.Vector2(float(distance),float(HAUTEUR)))
        self.canevas.clear(VERT)
        self.canevas.draw(self.textePts)
        self.canevas.draw(self.texteCoups)
        self.canevas.draw(self.cible)
        self.canevas.display()

    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction place la cible sur l'écran.  On l'appelle
            aussitôt que la distance de la cible est connue.

    \param[in] distance : la distance de la cible

    <b>Sorties:</b>
        \li (\c ecran)  La fenetre d'affichage  (\c graphique)
        \li (\c ecran)  La cible   (\c graphique)
        \li (\c ecran)  Le nombre de points courant de la cible  (\c entier)
        \li (\c ecran)  Le nombre de coups tires (0)  (\c entier)
    ** *********************************************************************'''
    def afficherPoints(self, points):

        #On genere le texte
        temp = str(points)
        self.textePts= sf.graphics.Text("Points restants = " + temp, self.font, 30 )
        self.textePts.style = sf.graphics.Text.BOLD
        self.textePts.color = sf.graphics.Color.RED
        self.textePts.position = (800,0)
        # Réactive l'écran pour remettre les couleurs et enlever le "fading"
        for event in self.canevas.events:
            # close canevas: exit
            if type(event) is sf.window.CloseEvent:
                    self.canevas.close()
        #--------------------------------------
        #on efface le contenu avant de dessiner
        self.canevas.clear(VERT)
        self.canevas.draw(self.textePts)
        self.canevas.draw(self.texteCoups)
        self.canevas.draw(self.cible)
        self.canevas.display()


    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction place la cible sur l'écran.  On l'appelle
            aussitôt que la distance de la cible est connue.

    \param[in] distance : la distance de la cible

    <b>Sorties:</b>
        \li (\c ecran)  La fenetre d'affichage  (\c graphique)
        \li (\c ecran)  La cible   (\c graphique)
        \li (\c ecran)  Le nombre de points courant de la cible  (\c entier)
        \li (\c ecran)  Le nombre de coups tires (0)  (\c entier)
    ** *********************************************************************'''
    def afficherCoups(self, coups):

        # On genere le texte
        self.texteCoups = sf.graphics.Text("Nombre de coups = " + str(coups) , self.font, 30 )
        self.texteCoups.style = sf.graphics.Text.BOLD
        self.texteCoups.color = sf.graphics.Color.RED
        self.texteCoups.position = (800,40)
        # Réactive l'écran pour remettre les couleurs et enlever le "fading"
        for event in self.canevas.events:
            # close window: exit
            if type(event) is sf.window.Event:
                    self.canevas.clear(VERT)
 
        #--------------------------------------
        #on efface le contenu avant de dessiner
        self.canevas.clear(VERT)
        self.canevas.draw(self.textePts)
        self.canevas.draw(self.texteCoups)
        self.canevas.draw(self.cible)
        self.canevas.display()

    '''* *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction affiche une animation pour le tir d'un
           boulet de canon

    \param[in] distance : la distance de la cible
    \param[in] vitesse : la vitesse initiale du boulet
    \param[in] radian : l'angle initial du canon

    <b>Sorties:</b>
        \li (\c ecran)  La fenetre d'affichage  (\c graphique)
        \li (\c ecran)  La cible   (\c graphique)
        \li (\c ecran)  Le nombre de points courant de la cible  (\c entier)
        \li (\c ecran)  Le nombre de coups tires (0)  (\c entier)
    ** *********************************************************************'''
    def afficherTir(self, distance, vitesse, radian):

        temp = vitesse*math.sin(radian);

        # on calcule la dure de la trajectoire du boulet
        duree = (temp/32.2) + (math.sqrt(temp*temp + (2*32.2*0))/32.2);
        t=0
        #on fait l'animation de la trajectoire du boulet
        while t <= duree :
            t+=.1
            # calcule la position du boulet au temps t
            posX= self.calculerX(radian, vitesse, t)
            posY = self.calculerY(0, radian, vitesse, t)
            #// On affiche seulement 30 images par seconde
            sf.system.sleep(sf.system.milliseconds(10))
            #// on s'assure que le boulet reste dans l'ecran
            if (posY < 00): posY = 0;
            pos = 800 -(posY+100)
            #// on initialise sa position
            self.boulet.position = (posX,pos)
            for event in self.canevas.events:
                # close window: exit
                if type(event) is sf.window.Event:
                    self.canevas.clear(VERT)
            # on l'affiche
            #on efface le contenu avant de dessiner
            self.canevas.clear(VERT)
            self.canevas.draw(self.boulet)
            self.canevas.draw(self.textePts)
            self.canevas.draw(self.texteCoups)
            self.canevas.draw(self.cible)
            self.canevas.display()

        self.imageCourante.position = (distance,HAUTEUR)
        #--------------------------------------------------------
        # On genere une animation d'explosion lorsque le boulet touche au sol
        # il y a 16 images dans l'animation
        for i in range(16):
            #on affiche environ 30 images par secondes
            sf.system.sleep(sf.system.milliseconds(50))
            # on recherche la bonne image
            self.imageCourante.texture_rectangle = self.images[i]
            # Réactive l'écran pour remettre les couleurs et enlever le "fading"
            for event in self.canevas.events:
                # close window: exit
                if type(event) is sf.window.Event:
                    self.canevas.clear(VERT)
            #--------------------------------------
            # On dessine
            self.canevas.clear(VERT)
            self.canevas.draw(self.textePts)
            self.canevas.draw(self.texteCoups)
            self.canevas.draw(self.cible)
            self.canevas.draw(self.imageCourante)
            self.canevas.display()


    ''' *********************************************************************
    \author Gabriel Girard
    \date 18 septembre 2015

    \brief Cette fonction place la cible sur l'écran.  On l'appelle
            aussitôt que la distance de la cible est connue.

    \param[in] distance : la distance de la cible

    <b>Sorties:</b>
        \li (\c ecran)  La fenetre d'affichage  (\c graphique)
        \li (\c ecran)  La cible   (\c graphique)
        \li (\c ecran)  Le nombre de points courant de la cible  (\c entier)
        \li (\c ecran)  Le nombre de coups tires (0)  (\c entier)
    ** *********************************************************************'''
    def afficherCote(self, cote) :
        # On genere le texte
        self.texteCote = sf.graphics.Text("Votre resultat : " + cote , self.font, 50 )
        self.texteCote.style = sf.graphics.Text.BOLD
        self.texteCote.color = sf.graphics.Color.RED
        self.texteCote.position = (100,400)
        # Réactive l'écran pour remettre les couleurs et enlever le "fading"
        for event in self.canevas.events:
            # close window: exit
            if type(event) is sf.window.Event:
                    self.canevas.clear(VERT)
         #//--------------------------------------
        #// on efface le contenu avant de dessiner
        self.canevas.clear(VERT)
        self.canevas.draw(self.textePts)
        self.canevas.draw(self.texteCoups)
        self.canevas.draw(self.texteCote)
        self.canevas.draw(self.cible)
        self.canevas.display()
